// Copyright (C) 2012 by Giandomenico De Sanctis
// gidesay@yahoo.it

/// cvGPU.h
/// C style interface to OpenCV GPU dynamic library (DLL) header file.

#ifndef __OPENCV_CV_GPU_H__
#define __OPENCV_CV_GPU_H__



#define C_EXTERN extern "C"

#ifdef _USRDLL
#define CVDLL_API __declspec(dllexport)
#else
#define CVDLL_API 
#endif

#include "opencv2/gpu/gpu.hpp"



/* Return the number of Cuda enabled devices, maybe zero */
C_EXTERN CVDLL_API int cvGPUGetCudaEnabledDeviceCount();

/* Load an image into a GPU matrix    */
C_EXTERN CVDLL_API int  cvGPUCreateImage( const CvArr* image );

/* Release a GPU matrix associated with an image    */
C_EXTERN CVDLL_API void  cvGPUReleaseImage( const CvArr* image );

/* Activate or deactivate the download of GPU result to CvArr struct   */
C_EXTERN CVDLL_API CDECL void cvGPUDownloadResult(BOOLEAN flg); 

/* Download a GPU matrix into the associated CvArr struct   */
C_EXTERN CVDLL_API CDECL void cvGPUReadImage(const CvArr* image); 

// release all memory areas allocated 
C_EXTERN CVDLL_API CDECL void cvGPURelease(); 

/* ---------------------------------------------------------------------------------------- */
/* Converts input array pixels from one color space to another */
C_EXTERN CVDLL_API void  cvGPUCvtColor( const CvArr* src, CvArr* dst, int code );

/* dilates input image (applies maximum filter) one or more times.
   If element pointer is NULL, 3x3 rectangular element is used */
C_EXTERN CVDLL_API void  cvGPUDilate( const CvArr* src, CvArr* dst,
                       IplConvKernel* element CV_DEFAULT(NULL),
                       int iterations CV_DEFAULT(1) );

/* Performs complex morphological transformation */
C_EXTERN CVDLL_API void  cvGPUMorphologyEx( const CvArr* src, CvArr* dst,
                             CvArr* temp, IplConvKernel* element,
                             int operation, int iterations CV_DEFAULT(1) );

/* Resizes image (input array is resized to fit the destination array) 
   Supports CV_INTER_NN, CV_INTER_LINEAR
   supports CV_8UC1, CV_8UC4 types */
C_EXTERN CVDLL_API void cvGPUResize( const CvArr* srcarr, CvArr* dstarr, int method );

/* Applies fixed-level threshold to grayscale image.
   This is a basic operation applied before retrieving contours */
C_EXTERN CVDLL_API double  cvGPUThreshold( const CvArr*  src, CvArr*  dst,
                            double  threshold, double  max_value,
                            int threshold_type );

/* Performs generic geometric transformation using the specified coordinate maps */
/* supports CV_8UC1, CV_8UC3 source types and CV_32FC1 map type */
C_EXTERN CVDLL_API void  cvGPURemap( const CvArr* src, CvArr* dst,
                      const CvArr* mapx, const CvArr* mapy,
                      int flags CV_DEFAULT(CV_INTER_LINEAR),
                      void* fillval CV_DEFAULT(0) );



#endif /* __OPENCV_CV_GPU_H__ */

