#include "stdafx.h"

#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc_c.h"

#include "../cvGPU.h"
#include <stdlib.h>
#include <stdio.h>

#include <iostream>


//using namespace cv;
//using namespace cv::gpu;

void help()
{

printf("\nTest on cvGPU Dll\n"
	"Call:\n  txCvGPU  [image]\n");
}

IplImage* iplSrc, *iplSrc4, *iplDst, *iplDstRes;
const int NCYCLES = 100;


int main( int argc, char** argv )
{
    char* filename = argc == 2 ? argv[1] : (char*)"test.jpg";

    if( (iplSrc = cvLoadImage(filename,1)) == 0 )
        return help(), -1;

    help();

	std::cout << "Cuda enabled devices: " << cvGPUGetCudaEnabledDeviceCount() <<"\n" ;

	iplDst = cvCreateImage(cvGetSize(iplSrc),8,1);
	cvZero(iplDst);
	IplImage* iplDst2 = cvCloneImage(iplDst);
	IplImage* iplDst3 = cvCloneImage(iplDst);

	iplSrc4 = cvCreateImage(cvGetSize(iplSrc),8,4);
	cvZero(iplSrc4);

	iplDstRes = cvCreateImage(cvSize((iplSrc->width >> 1),(iplSrc->height >> 1)) ,8,4);
	cvZero(iplDstRes);


	//{ Precise measurement of time in ms }
	LONGLONG Freq = 0;
	LONGLONG StartCount = 0;
	LONGLONG StopCount = 0;
	float TimingSeconds = 0.0;

	// Cpu
	QueryPerformanceFrequency((LARGE_INTEGER*) &Freq);
	QueryPerformanceCounter((LARGE_INTEGER*) &StartCount);

	for (int i=0; i<NCYCLES; i++)
	{
		std::cout << "Cpu Count: " << i << "\n";
//		cvCvtColor(iplSrc, iplSrc4, CV_BGR2BGRA); 

//		cvResize(iplSrc, iplDst, CV_INTER_LINEAR); // work directly on 3 channels

//		cvDilate(iplDst, iplDst2, 0, 3);

		cvCvtColor(iplSrc, iplDst, CV_BGR2GRAY); 
		cvThreshold(iplDst, iplDst2, 125.0, 255.0, CV_THRESH_BINARY);
		cvThreshold(iplDst, iplDst3, 80.0, 255.0, CV_THRESH_TOZERO);

	};
    QueryPerformanceCounter((LARGE_INTEGER*) &StopCount);
	// msec
	TimingSeconds = 1000 * (StopCount - StartCount) / Freq;
	std::cout << "Cpu time msec: " << TimingSeconds << "\n";

	Sleep(5000);


	// GPU 
	// first call to a GPU function has a big delay, caused by initialization on Cuda
	cvGPUCreateImage(iplSrc);
	cvZero(iplSrc4);
	cvZero(iplDstRes);
	cvZero(iplDst);

	QueryPerformanceFrequency((LARGE_INTEGER*) &Freq);
	QueryPerformanceCounter((LARGE_INTEGER*) &StartCount);

	cvGPUDownloadResult(FALSE);

	for (int i=0; i<NCYCLES; i++)
	{
		std::cout << "GPU Count: " << i << "\n";
//		cvGPUCvtColor(iplSrc, iplSrc4, CV_BGR2BGRA); // convert to 4 channels
//		cvGPUResize(iplSrc4, iplDstRes, CV_INTER_LINEAR);

		cvGPUCvtColor(iplSrc, iplDst, CV_BGR2GRAY); 
		cvGPUThreshold(iplDst, iplDst2, 125.0, 255.0, CV_THRESH_BINARY);
		cvGPUThreshold(iplDst, iplDst3, 80.0, 255.0, CV_THRESH_TOZERO);

//  		cvGPUDilate(iplDst, iplDst2, 0, 3);
	};
	cvGPUDownloadResult(TRUE);
	cvGPUReadImage(iplDst);
	cvGPUReadImage(iplDst2);
	cvGPUReadImage(iplDst3);
//	cvGPUReadImage(iplDstRes);

	cvGPURelease();

    QueryPerformanceCounter((LARGE_INTEGER*) &StopCount);
	// msec
	TimingSeconds = 1000 * (StopCount - StartCount) / Freq;
	std::cout << "GPU time msec: " << TimingSeconds;

	Sleep(5000);

    cvSaveImage("testOut-GPU.jpg", iplDst);

	cvSaveImage("testOut2-GPU.jpg", iplDst2);

	cvSaveImage("testOut3-GPU.jpg", iplDst3);

//	cvSaveImage("testResized-GPU.jpg", iplDstRes);

	return 0;
}
